package com.extensibility.util;

import java.util.NoSuchElementException;

public final class Deque {
   private static final int DEFAULT_SIZE = 32;
   Object[] array;
   int head;
   int tail;
   int size;

   public Deque() {
      this.array = new Object[32];
      this.head = 16;
      this.tail = this.head - 1;
      this.size = 0;
   }

   public Deque(int var1) {
      this.array = new Object[var1];
      this.head = var1 / 2;
      this.tail = this.head - 1;
      this.size = 0;
   }

   public int size() {
      return this.size;
   }

   public void addHead(Object var1) {
      if (this.size == this.array.length) {
         this.grow();
      }

      this.head = this.decrement(this.head);
      this.array[this.head] = var1;
      ++this.size;
   }

   public void addTail(Object var1) {
      if (this.size == this.array.length) {
         this.grow();
      }

      this.tail = this.increment(this.tail);
      this.array[this.tail] = var1;
      ++this.size;
   }

   public Object removeHead() {
      if (this.size == 0) {
         throw new NoSuchElementException();
      } else {
         Object var1 = this.array[this.head];
         this.head = this.increment(this.head);
         --this.size;
         return var1;
      }
   }

   public Object removeTail() {
      if (this.size == 0) {
         throw new NoSuchElementException();
      } else {
         Object var1 = this.array[this.tail];
         this.tail = this.decrement(this.tail);
         --this.size;
         return var1;
      }
   }

   public Object peekHead() {
      if (this.size == 0) {
         throw new NoSuchElementException();
      } else {
         Object var1 = this.array[this.head];
         return var1;
      }
   }

   public Object peekTail() {
      if (this.size == 0) {
         throw new NoSuchElementException();
      } else {
         Object var1 = this.array[this.tail];
         return var1;
      }
   }

   private int increment(int var1) {
      return (var1 + 1) % this.array.length;
   }

   private int decrement(int var1) {
      return (var1 - 1 + this.array.length) % this.array.length;
   }

   private void grow() {
      Object[] var1 = new Object[this.array.length * 3 / 2];
      int var2 = 0;

      for(int var3 = this.head; var2 < this.size; var3 = this.increment(var3)) {
         var1[var2] = this.array[var3];
         ++var2;
      }

      this.head = 0;
      this.tail = this.size - 1;
      this.array = var1;
   }
}
