package com.extensibility.util.coll;

import com.sun.java.util.collections.AbstractList;
import com.sun.java.util.collections.ArrayList;
import com.sun.java.util.collections.Iterator;
import com.sun.java.util.collections.ListIterator;

public class BigList extends AbstractList {
   protected static int DEFAULT_MAX_CHUNK_SIZE = 50;
   protected int maxChunkSize;
   protected ArrayList chunkArray;
   protected int size = 0;
   protected int lastResult = 0;

   public BigList() {
      this.maxChunkSize = DEFAULT_MAX_CHUNK_SIZE;
      this.chunkArray = new ArrayList();
   }

   public BigList(int var1) {
      this.maxChunkSize = DEFAULT_MAX_CHUNK_SIZE;
      this.chunkArray = new ArrayList(var1 / this.maxChunkSize + 1);
   }

   public BigList(int var1, int var2) {
      this.maxChunkSize = var2;
      this.chunkArray = new ArrayList(var1 / var2 + 1);
   }

   public int size() {
      return this.size;
   }

   public boolean contains(Object var1) {
      return ((AbstractList)this).indexOf(var1) >= 0;
   }

   public boolean add(Object var1) {
      this.add(this.size(), var1);
      return true;
   }

   public Object get(int var1) {
      this.rangeCheck(var1);
      ListChunk var2 = this.findChunk(var1);
      return ((ArrayList)var2).get(var1 - var2.startListIndex);
   }

   public Object set(int var1, Object var2) {
      this.rangeCheck(var1);
      ListChunk var3 = this.findChunk(var1);
      Object var4 = ((ArrayList)var3).get(var1 - var3.startListIndex);
      this.noteElementLocation(var4, (ListChunk)null);
      ((ArrayList)var3).set(var1 - var3.startListIndex, var2);
      this.noteElementLocation(var2, var3);
      return var4;
   }

   public void add(int var1, Object var2) {
      if (var1 >= 0 && var1 <= this.size) {
         ++super.modCount;
         ListChunk var3 = this.findChunkForInsertion(var1);
         if (((ArrayList)var3).size() == this.maxChunkSize) {
            int var4 = var1 - var3.startListIndex;
            ListChunk var5 = new ListChunk(var1, this.maxChunkSize);
            this.splitListChunk(var3, var5, var4);
            this.insertListChunk(var5, var3.chunkIndex + 1);
            if (var4 == this.maxChunkSize) {
               var3 = var5;
            }
         }

         int var6 = var1 - var3.startListIndex;
         ((ArrayList)var3).add(var6, var2);
         this.adjustStartIndices(var3.chunkIndex + 1, 1);
         this.noteElementLocation(var2, var3);
      } else {
         throw new IndexOutOfBoundsException();
      }
   }

   public Object remove(int var1) {
      this.rangeCheck(var1);
      ++super.modCount;
      ListChunk var2 = this.findChunk(var1);
      Object var3 = ((ArrayList)var2).get(var1 - var2.startListIndex);
      this.noteElementLocation(var3, (ListChunk)null);
      ((ArrayList)var2).remove(var1 - var2.startListIndex);
      this.adjustStartIndices(var2.chunkIndex + 1, -1);
      if (((ArrayList)var2).size() == 0) {
         this.removeEmptyListChunk(var2);
      }

      return var3;
   }

   public boolean remove(Object var1) {
      int var2 = ((AbstractList)this).indexOf(var1);
      if (var2 >= 0) {
         this.remove(var2);
      }

      return var2 >= 0;
   }

   protected ListChunk findChunk(int var1) {
      if (this.lastResult < this.chunkArray.size()) {
         ListChunk var2 = (ListChunk)this.chunkArray.get(this.lastResult);
         if (var1 >= var2.startListIndex && var1 < var2.startListIndex + ((ArrayList)var2).size()) {
            return var2;
         }
      }

      int var6 = 0;
      int var3 = this.chunkArray.size();

      while(var6 < var3) {
         int var4 = (var6 + var3) / 2;
         ListChunk var5 = (ListChunk)this.chunkArray.get(var4);
         if (var1 < var5.startListIndex) {
            var3 = var4;
         } else {
            if (var1 < var5.startListIndex + ((ArrayList)var5).size()) {
               this.lastResult = var4;
               return var5;
            }

            var6 = var4 + 1;
         }
      }

      throw new RuntimeException("findChunk failed.");
   }

   protected ListChunk findChunkForInsertion(int var1) {
      ListChunk var2;
      if (var1 == this.size) {
         if (this.size == 0) {
            this.chunkArray.add(new ListChunk(var1, this.maxChunkSize));
         }

         var2 = (ListChunk)this.chunkArray.get(this.chunkArray.size() - 1);
      } else {
         var2 = this.findChunk(var1);
         if (var1 == var2.startListIndex && var2.chunkIndex != 0) {
            var2 = (ListChunk)this.chunkArray.get(var2.chunkIndex - 1);
         }
      }

      return var2;
   }

   protected void splitListChunk(ListChunk var1, ListChunk var2, int var3) {
      int var4 = ((ArrayList)var1).size();

      for(int var5 = var3; var5 < var4; ++var5) {
         Object var6 = ((ArrayList)var1).get(var5);
         this.noteElementLocation(var6, var2);
         ((ArrayList)var2).add(var6);
      }

      for(int var7 = var4 - 1; var7 >= var3; --var7) {
         ((ArrayList)var1).remove(var7);
      }

   }

   protected void insertListChunk(ListChunk var1, int var2) {
      this.chunkArray.add(var2, var1);
      var1.chunkIndex = var2;
      int var3 = this.chunkArray.size();

      for(int var4 = var2 + 1; var4 < var3; ++var4) {
         ListChunk var5 = (ListChunk)this.chunkArray.get(var4);
         ++var5.chunkIndex;
      }

   }

   protected void adjustStartIndices(int var1, int var2) {
      int var3 = this.chunkArray.size();

      for(int var4 = var1; var4 < var3; ++var4) {
         ListChunk var5 = (ListChunk)this.chunkArray.get(var4);
         var5.startListIndex += var2;
      }

      this.size += var2;
   }

   protected void removeEmptyListChunk(ListChunk var1) {
      this.chunkArray.remove(var1.chunkIndex);
      int var2 = this.chunkArray.size();

      for(int var3 = var1.chunkIndex; var3 < var2; ++var3) {
         var1 = (ListChunk)this.chunkArray.get(var3);
         --var1.chunkIndex;
      }

   }

   protected void removeRange(int var1, int var2) {
      if (var1 < var2) {
         ++super.modCount;
      }

      int var4;
      for(; var1 < var2; var2 = var4) {
         ListChunk var3 = this.findChunk(var2 - 1);
         var4 = Math.max(var3.startListIndex, var1);

         for(int var5 = var2 - 1; var5 >= var4; --var5) {
            Object var6 = ((ArrayList)var3).get(var5 - var3.startListIndex);
            this.noteElementLocation(var6, (ListChunk)null);
            ((ArrayList)var3).remove(var5 - var3.startListIndex);
         }

         this.adjustStartIndices(var3.chunkIndex + 1, var4 - var2);
         if (((ArrayList)var3).size() == 0) {
            this.removeEmptyListChunk(var3);
         }
      }

   }

   private void rangeCheck(int var1) {
      if (var1 >= this.size || var1 < 0) {
         throw new IndexOutOfBoundsException(String.valueOf(String.valueOf(String.valueOf("Index: ").concat(String.valueOf(var1))).concat(String.valueOf(", Size: "))).concat(String.valueOf(this.size)));
      }
   }

   protected void noteElementLocation(Object var1, ListChunk var2) {
   }

   public Iterator iterator() {
      return new Itr(this);
   }

   public ListIterator listIterator() {
      return this.listIterator(0);
   }

   public ListIterator listIterator(int var1) {
      if (var1 >= 0 && var1 <= this.size()) {
         return new ListItr(this, var1);
      } else {
         throw new IndexOutOfBoundsException(String.valueOf("Index: ").concat(String.valueOf(var1)));
      }
   }

   private int getModCount() {
      return super.modCount;
   }

   // $FF: synthetic method
   static int access$3000071(BigList var0) {
      return var0.getModCount();
   }
}
